//////////////////////////////////////////////////////////////////////////////////////////
//                                                                                      //
//  Boardmodul: Bereitstellung gertespezifischer Funktionen und Mechanismen            //
//  _________________________________  													//
//                                                                                      //
//  Copyright (c) 2017 Reimar Grasbon                                                   //
//                                                                                      //
//////////////////////////////////////////////////////////////////////////////////////////
//                                                                                      //
//  This software is subjected to the FreeBSD License. Redistribution and use in source //
//  and binary forms, with or without modification, are permitted provided that the     //
//  following conditions are met:                                                       //
//                                                                                      //
//  1. Redistributions of source code must retain the above copyright notice, this list //
//     of conditions and the following disclaimer.                                      //
//                                                                                      //
//  2. Redistributions in binary form must reproduce the above copyright notice, this   //
//     list of conditions and the following disclaimer in the documentation and/or      //
//     other materials provided with the distribution.                                  //
//                                                                                      //
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY //
//  EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED           //
//  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  //
//  IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,    //
//  INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT  //
//  NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR  //
//  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,   //
//  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)  //
//  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE          //
//  POSSIBILITY OF SUCH DAMAGE.                                                         //
//                                                                                      //
//////////////////////////////////////////////////////////////////////////////////////////

#pragma once

#include <avr/wdt.h>
#include <avr/eeprom.h>
#include "AVRChip.h"
#include "Conf.h"
#include "TypeDefs.h"

//////////////////////////////////////////////////////////////////////////////////////////
#define TIMERTICKS			  	TCNT1
#define MS_TO_TIMER_TICKS(ms) 	(((F_CPU/1024)*ms)/1000)

//////////////////////////////////////////////////////////////////////////////////////////
// Funktionen ////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////
void	BOARD_Setup(void);
void	BOARD_Reset(void);
void	BOARD_EnterAppl(void);
void	BOARD_TransmitByte(UInt08);
bool	BOARD_WaitForByte(UInt08*, UInt08);
#define BOARD_Watchdog()			    wdt_reset()
#define BOARD_SetTimeout(ms)   		    (TIMERTICKS + MS_TO_TIMER_TICKS(ms))
#define BOARD_Timeout(toHdl) 		    (((SInt16)(TIMERTICKS - toHdl)) > 0)
#define BOARD_EEPLoad(adr, mem, size)   eeprom_read_block(mem, (void*)adr, size)
#define BOARD_EEPSave(adr, mem, size)	eeprom_update_block(mem, (void*)adr, size)
#if CFG_LED_LOW_ACTIVE == true
#define BOARD_SetLed()				    ClrPin(CFG_SYSTEM_LED)
#else
#define BOARD_SetLed()				    SetPin(CFG_SYSTEM_LED)
#endif
#define BOARD_InvLed()				    InvPin(CFG_SYSTEM_LED)

//////////////////////////////////////////////////////////////////////////////////////////
