﻿//////////////////////////////////////////////////////////////////////////////////////////
//                                                                                      //
//      The Cy4NET System                      Version 19.09                            //
//      __________________________             _________________________________        //
//                                                                                      //
//      Decentralised Realtime                 Copyright (c) 2012 Reimar Grasbon        //
//      Peer-to-peer Communication             (www.cy4net.org)                         //
//      Protocol                                                                        //
//      _________________________              _________________________________        //
//                                                                                      //
//      [Address]                                                                       //
//                                                                                      //
//////////////////////////////////////////////////////////////////////////////////////////
//                                                                                      //
//  This software is subjected to the FreeBSD License. Redistribution and use in source //
//  and binary forms, with or without modification, are permitted provided that the     //
//  following conditions are met:                                                       //
//                                                                                      //
//  1. Redistributions of source code must retain the above copyright notice, this list //
//     of conditions and the following disclaimer.                                      //
//                                                                                      //
//  2. Redistributions in binary form must reproduce the above copyright notice, this   //
//     list of conditions and the following disclaimer in the documentation and/or      //
//     other materials provided with the distribution.                                  //
//                                                                                      //
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY //
//  EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED           //
//  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  //
//  IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,    //
//  INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT  //
//  NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR  //
//  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,   //
//  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)  //
//  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE          //
//  POSSIBILITY OF SUCH DAMAGE.                                                         //
//                                                                                      //
//////////////////////////////////////////////////////////////////////////////////////////
//                                                                                      //
//  Cy4NET-Adresse                                                                      //
//  --------------                                                                      //
//                                                                                      //
//                                                                                      //
//                    +-----+  +-----+                                                  //
//      Cy4Address:   | NET |  | DEV |                                                  //
//                    +-----+  +-----+                                                  //
//                                                                                      //
//                                                                                      //
//      Eine Cy4NET-Adresse setzt sich aus zwei Komponenten zusammen, der Netzadresse   //
//      <NET> und der Geräteadresse <DEV>. Jede Komponente für sich kann Werte von 0    //
//      bis 255 annehmen, wobei Adresse 255.255 (NET.DEV) als Ausprägung für "ungültige //
//      Adresse" vorbelegt ist und Adresswert 0 für <NET> oder <DEV> als Rundrufadresse //
//      (Broadcast) fungiert.                                                           //
//                                                                                      //
//      Hintergrund der Aufteilung in Netz- und Geräteadresse ist, das Cy4-Netzwerk     //
//      in Subnetze segmentieren zu können. Mitglieder eines solchen Subnetzes können   //
//      beispielsweise durch einen subnetzinternen Rundruf gemeinschaftlich ange-       //
//      sprochen werden (Broadcast-Domänen). Darüber hinaus ist es möglich durch        //
//      Einsatz geeigneter Router die Buslast zu reduziert, indem subnetzlokale         //
//      Kommunikation nicht mehr nach außerhalb weitergereicht wird.                    //
//                                                                                      //
//                    +--------+--- ... ---+--------+--- ... ---+                       //
//                    |        |           |        |           |                       //
//                 +--+--+  +--+--+     +--+--+  +--+--+     +--+--+                    //
//                 | 1.1 |  | 1.2 | ... | 2.1 |  | 2.2 | ... | 3.* |                    //
//                 +-----+  +-----+     +-----+  +-----+     +--+--+                    //
//                                                              |                       //
//                                                        +--- ... ----+                //
//                                                        |            |                //
//                                                     +--+--+     +---+---+            //
//                                                     | 3.1 | ... | 3.255 |            //
//                                                     +-----+     +-------+            //
//                                                                                      //
//      Der Adressraum eines Cy4-Netzwerks umfaßt theoretische 255*255-1 = 65024        //
//      Adressen.                                                                       //
//                                                                                      //
//////////////////////////////////////////////////////////////////////////////////////////

#include "Cy4Address.h"

//////////////////////////////////////////////////////////////////////////////////////////
//  Funktionen  //////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////
Cy4Address
CY4ADDRESS_Create(UInt08 net, UInt08 dev)
{
    Cy4Address address = CY4ADDRESS(net, dev);
    return address;
}

//////////////////////////////////////////////////////////////////////////////////////////
UInt08
CY4ADDRESS_Net(const Cy4Address* address)
{
    return address->net;
}

//////////////////////////////////////////////////////////////////////////////////////////
UInt08
CY4ADDRESS_Dev(const Cy4Address* address)
{
    return address->dev;
}

//////////////////////////////////////////////////////////////////////////////////////////
bool
CY4ADDRESS_IsValid(const Cy4Address* address)
{
    return address->net != 0xFF || address->dev != 0xFF;
}

//////////////////////////////////////////////////////////////////////////////////////////
bool
CY4ADDRESS_IsBroadcast(const Cy4Address* address)
{
    return address->net == 0 || address->dev == 0;
}

//////////////////////////////////////////////////////////////////////////////////////////
