﻿//////////////////////////////////////////////////////////////////////////////////////////
//                                                                                      //
//      The Cy4NET System                      Version 19.09                            //
//      __________________________             _________________________________        //
//                                                                                      //
//      Decentralised Realtime                 Copyright (c) 2012 Reimar Grasbon        //
//      Peer-to-peer Communication             (www.cy4net.org)                         //
//      Protocol                                                                        //
//      _________________________              _________________________________        //
//                                                                                      //
//      [Cy4 Type Plate]                                                                //
//                                                                                      //
//////////////////////////////////////////////////////////////////////////////////////////
//  This software is subjected to the FreeBSD License. Redistribution and use in source //
//                                                                                      //
//  and binary forms, with or without modification, are permitted provided that the     //
//  following conditions are met:                                                       //
//                                                                                      //
//  1. Redistributions of source code must retain the above copyright notice, this list //
//     of conditions and the following disclaimer.                                      //
//                                                                                      //
//  2. Redistributions in binary form must reproduce the above copyright notice, this   //
//     list of conditions and the following disclaimer in the documentation and/or      //
//     other materials provided with the distribution.                                  //
//                                                                                      //
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY //
//  EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED           //
//  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  //
//  IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,    //
//  INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT  //
//  NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR  //
//  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,   //
//  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)  //
//  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE          //
//  POSSIBILITY OF SUCH DAMAGE.                                                         //
//                                                                                      //
//////////////////////////////////////////////////////////////////////////////////////////
//                                                                                      //
//  Cy4NET Geräte-Typenschild                                                           //
//                                                                                      //
//  Ein Cy4NET Gerät ist stets eine Kombination aus Anwendung und Hardwareplattform.    //
//  Die Anwendung wie auch die Hardwareplattform sind dabei bestimmt durch ihre Typ-    //
//  nummern. Durch sie sind Eigenschaften und Komponenten eindeutig festgelegt. Wird    //
//  eine Eigenschaft verändert oder erweitert, verändert das den Typ und damit auch die //
//  Typnummer. Die Nummern identifizieren zwar eindeutig Anwendung und Board, sind      //
//  aber unabhängig voneinander zu betrachten. Beispielsweise wäre es denkbar, dass     //
//  eine Anwendung des Typs 123 auf Boards der Typen 1000, 1001 und 1003 zum Einsatz    //
//  kommt. Umgekehrt könnte es auch sein, dass z.B. ein Board des Typs 777 für Anwend-  //
//  ungen der Typen 200, 201, und 300 einsetzbar ist. Die Kombination aus Anwendungs-   //
//  und Boardtypennummer legt also letzten Endes das Cy4NET Gerät fest. Das ist zu      //
//  berücksichtigen, wenn Anwendungen eines bestimmten Typs für mehrere Zielplattformen //
//  erstellt werden. Jedes Board benötigt seine individuelle Variante der Anwendung.    //
//  Im Beispiel oben wären das z.B. 123/1000, 123/1001 und 123/1003 bzw. 200/777,       //
//  201/777 und 300/777.                                                                //
//  Diese eindeutige Kennung ist in jeder Cy4NET Gerätefirmware hinterlegt kann abge-   //
//  fragt werden.                                                                       //
//                                                                                      //
//                  Beispiel Geräte-Typenkennung:                                       //
//                                                                                      //
//                    Anwendungstypennummer: 123                                        //
//                    Anwendungsversion:     4.5                                        //
//                    Boardtypennummer:      6789                                       //
//                                                                                      //
//  Im Gegensatz zur Entwicklungsstufe der Software, wird die der Hardwareplattform     //
//  nicht ausgegeben. Grund dafür ist, dass die Firmware ohne weitere Maßnahmen keine   //
//  Möglichkeit hat, die zugrundeliegende Boardrevision abzufragen. Folglich ist die    //
//  Revisionierung der Hardware auf externe Maßnahmen angewiesen, z.B. als Markierung   //
//  auf dem PCB.                                                                        //
//                                                                                      //
//  Format                                                                              //
//  ------                                                                              //
//                                                                                      //
//  Anwendungs- und Boardtypennummern sind maximal fünfstellig und dürfen Werte von 1   //
//  bis 99999 annehmen, also alle Werte außer 0. Die Anwendungsversion besteht aus      //
//  zwei dezimalen Ziffern, bei denen die erste die Hauptversionsnummer (Majornumber)   //
//  und die zweite die Unternummer (Minornumber) der Anwendung beschreibt. Dargestellt  //
//  wird die Versionsnummer in der üblichen Form Major.Minor. Auch hier ist 0.0 nicht   //
//  erlaut, so dass sich die Anwendungsversion von 0.1 bis 9.9 erstrecken kann.         //
//                                                                                      //
//  Typenschild                                                                         //
//  -----------                                                                         //
//                                                                                      //
//  Abrufbar sind diese, als Typenschild bezeichneten Informationen, mittels des        //
//  Dienstes CallTypePlate (FFFFF0:7:RO). Bei CallTypePlate handelt es sich um einen    //
//  Pflichtdienst den jedes Cy4NET Gerät bereitzustellen hat. Es ist ein "nur-lese"     //
//  Dienst der sieben Bytes umfaßt und die Informationen des Typenschildes in folgender //
//  Kodierung bereitstellt:                                                             //
//                                                                                      //
//                          0   1   2   3   4   5   6                                   //
//                        +---+---+---+---+---+---+---+   A = Anwendungstypennummer     //
//                        | A | A | A | V | B | B | B |   V = Anwendungsversion         //
//                        +---+---+---+---+---+---+---+   B = Boardtypennummer          //
//                                                                                      //
//  Die ersten 3 Bytes repräsentieren die Anwendungstypennummer. Im 4. Byte liegt die   //
//  Anwendungsversion und in den restlichen 3 Bytes ist die Boardtypennummer unterge-   //
//  gebracht. Alle Einträge sind "big-endian" BCD kodiert, d.h. das erste Halbbyte hat  //
//  die höchstwertigste Stelle und alle Nummern sind als hexadezimal geschriebene Dezi- //
//  malzahlen zu interpretieren. Anwendungs- und Boardtypennummern führen im unge-      //
//  nutzten höchstwertigste Halbbyte (fünfstellig) einen Präfix. Die Typennummer der    //
//  Anwendung beginnen mit A (hex), die Boardtypennummer mit B (hex). In den restlichen //
//  fünf Halbbytes ist dann die eigentliche Typennummer BCD kodiert untergebracht. Die  //
//  Präfixe sind folglich nicht Bestandteil der Nummern sondern dienen der klaren Zu-   //
//  ordnung. Die Versionsnummer hat keinen Präfix und ist direkt als zweistellige BCD-  //
//  Zahl abgelegt:                                                                      //
//                                                                                      //
//                  Beispiel:                                                           //
//                                                                                      //
//                          A0h, 01h, 23h, 45h, B0h, 67h und 89h                        //
//                                                                                      //
//                          Anwendungstypennummer: 123                                  //
//                          Anwendungsversion:     4.5                                  //
//                          Boardtypennummer:      6789                                 //
//                                                                                      //
//  Jegliche davon abweichene Kodierung ist kein gültiges Cy4-Typenschild.              //
//                                                                                      //
//                                                                                      //
//  Das Makro CY4TYPEPLATE                                                              //
//  ----------------------                                                              //
//                                                                                      //
//  Das Makro CY4TYPEPLATE initialisiert ein Byte-Feld mit den sieben passenden Bytes   //
//  des TypePlate-Dienstes:                                                             //
//                                                                                      //
//        Beispiel:                                                                     //
//                                                                                      //
//        const UInt08 typePlate[] = CY4TYPEPLATE(123, 45, 6789);                       //
//                                                                                      //
//       <typePlate> hat danach folgenden Inhalt: A0h,01h,23h,45h,B0h,67h,89h           //
//                                               |-----------|---|-----------|          //
//                                                     |       :       |                //
//                                                  A00123h   45h   B06789h             //
//                                                                                      //
//                                                                                      //
//  Reservierte Nummernbereiche                                                         //
//  ---------------------------                                                         //
//                                                                                      //
//  Cy4NET Geräte und Anwendungen sind in der Regel öffentlich zugänglich, folglich ist //
//  die Zuordnung der Nummern durch die Veröffentlichung festgelegt. Nicht jedoch der   //
//  Bereich von 90000 - 99999. Anwendungs- und Boardtypennummern aus diesem Bereich     //
//  sind nicht festgelegt. Jeder kann Nummern aus diesem Bereich nach individuellem     //
//  Gutdünken nutzen und vergeben. Damit ist das der bevorzugte Bereich für betriebs-   //
//  interne Geräte oder individuelle Bastelprojekte.                                    //
//  Eine weitere Einschränkung betrifft Anwendungstypennummern aus dem Bereich von 1    //
//  bis 99. Dieser Bereich ist für Programmlader-Module reserviert.                     //
//                                                                                      //
//////////////////////////////////////////////////////////////////////////////////////////

#pragma once

//////////////////////////////////////////////////////////////////////////////////////////
#define HEXIFY(X) PRE_OX(X) // Konkarteniert die Hexadezimalkennung 0x mit 'X',
#define PRE_OX(T) (0x ## T) // Beispiel: HEXIFY(1234) -> 0x1234

#define CY4TYPEPLATE(applicationType, applicationVers, boardType)             \
{   0xA0 + (unsigned char)(((unsigned long)HEXIFY(applicationType))  >> 16),  \
           (unsigned char)(((unsigned long)HEXIFY(applicationType))  >>  8),  \
           (unsigned char)(((unsigned long)HEXIFY(applicationType))  >>  0),  \
           (unsigned char)(                HEXIFY(applicationVers)),          \
    0xB0 + (unsigned char)(((unsigned long)HEXIFY(boardType)) >> 16),         \
           (unsigned char)(((unsigned long)HEXIFY(boardType)) >>  8),         \
           (unsigned char)(((unsigned long)HEXIFY(boardType)) >>  0),         \
}

//////////////////////////////////////////////////////////////////////////////////////////

